// diagram.cpp
/*
Copyright (C) 2004  Anders Hedstrom (grymse@alhem.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <mysql/mysql.h>
#include <libmysqlwrapped.h>
#include <libdbd.h>
#include <string>
#include <libcgi++.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include "dbd.h"
#include "html.h"
#include "utils.h"
#include "gsql.h"

#include "diagram.h"


std::string fieldtype(db::Flds *f)
{
	std::string tmp = f -> fieldstr;
// fieldtype 1-char,2-int,3-datetime,4-enum/options,5-ptr,6-number,7-set
	if (!tmp.size())
	{
		switch (f -> fieldtype)
		{
		case 1:
			tmp = "Char";
			break;
		case 2:
			tmp = "Int";
			break;
		case 3:
			tmp = "Datetime";
			break;
		case 4:
			tmp = "Enum";
			break;
		case 5:
			tmp = "Int(FK)";
			break;
		case 6:
			tmp = "Number";
			break;
		case 7:
			tmp = "Set";
			break;
		}
	}
	return tmp;
}


void create_dot_png(Database& db,Query& q,char *sql,db::Dbs *sdb)
{
	FILE *fil;
	char filename[200];
	sprintf(filename,"/tmp/%ld.dot",sdb -> num);
	if ((fil = fopen(filename,"wt")) != NULL)
	{
		Query q2(db);
/*
digraph "G" {
	rankdir=LR
	node [shape=record]
	aa [shape=record, label="<f0>hej|<f1>hopp:INT|<f2>num:VARCHAR|<f3>fk:INT"]
	ab [shape=record, label="<f0>hej|<f1>hopp:INT|<f2>num:VARCHAR|<f3>fk:INT"]
	ac [shape=record, label="<f0>hej|<f1>hopp:INT|<f2>num:VARCHAR|<f3>fk:INT"]
	ad [shape=record, label="<f0>hej|<f1>hopp:INT|<f2>num:VARCHAR|<f3>fk:INT"]

	aa:f3 -> ab:f1
	ac:f3 -> ad:f1
	ab:f3 -> ad:f1	
}
*/
		fprintf(fil,"digraph \"G\" {\n");
		fprintf(fil,"\trankdir=LR;\n");
//		fprintf(fil,"\trotate=90\n");
		fprintf(fil,"\trank=same\n");
		fprintf(fil,"\tratio=auto\n");
		fprintf(fil,"\tnode [shape=record,fontsize=9];\n");
		fprintf(fil,"\tedge [arrowtail=odiamond,arrowhead=none];\n");

		sprintf(sql,"select * from tbls where dbnum=%ld and deleted=0",sdb -> num);
		q.get_result(sql);
		while (q.fetch_row())
		{
			db::Tbls tbl(&db,&q);
			fprintf(fil,"\t%s [label=\"<f0>%s", tbl.name.c_str(), tbl.name.c_str());
			sprintf(sql,"select * from flds where tblnum=%ld and deleted=0 order by ordning",tbl.num);
			q2.get_result(sql);
			while (q2.fetch_row())
			{
				db::Flds fld(&db,&q2);
				fprintf(fil,"|<%s>%s:%s", fld.name.c_str(), fld.name.c_str(), fieldtype(&fld).c_str());
			}
			q2.free_result();
			fprintf(fil,"\""); // End of label
			fprintf(fil,",URL=\"/cgi-bin/dbd/dbd?seltbl=%ld\"",tbl.num); // end of table definition
			fprintf(fil,",tooltip=\"%s\"]\n",tbl.name.c_str());
		}
		q.free_result();

		sprintf(sql,"select * from tbls where dbnum=%ld and deleted=0",sdb -> num);
		q.get_result(sql);
		while (q.fetch_row())
		{
			db::Tbls tbl(&db,&q);
			sprintf(sql,"select * from flds where tblnum=%ld and deleted=0",tbl.num);
			q2.get_result(sql);
			while (q2.fetch_row())
			{
				db::Flds fld(&db,&q2);
				if (fld.fieldtype == 5) // ptrtotable
				{
					db::Tbls table(db,fld.ptrtotblnum);
					fprintf(fil,"\t%s:%s -> %s:num\n",tbl.name.c_str(),fld.name.c_str(),table.name.c_str());
				}
			}
			q2.free_result();
		}
		q.free_result();

		//
		fprintf(fil,"}\n");
		fclose(fil);
	}

	sprintf(filename,"cat /tmp/%ld.dot | %s -Tcmap -o /tmp/cmap%ld.html",sdb -> num,_DOTPATH,sdb -> num);
	system( filename );
	sprintf(filename,"cat /tmp/%ld.dot | %s -Tpng -o /tmp/image%ld.png",sdb -> num,_DOTPATH,sdb -> num);
	system( filename );
}


void generate_diagram(Database& db,Query& q,char *sql,db::Dbs *sdb,Cookies *cs)
{
	header0("Table relations diagram",1,cs);
	fflush(out);
	create_dot_png(db,q,sql,sdb);
	fprintf(out,"<img src=\"/cgi-bin/dbd/dbd?diagram=1\" alt=\"Table relations diagram\" usemap=\"map1\">");
	fprintf(out,"<map name=\"map1\">");
	char filename[200];
	sprintf(filename,"/tmp/cmap%ld.html",sdb -> num);
	FILE *fil = fopen(filename,"rt");
	if (fil)
	{
		char slask[4000];
		fgets(slask,4000,fil);
		while (!feof(fil))
		{
			fprintf(out,"%s\n",slask);
			fgets(slask,4000,fil);
		}
		fclose(fil);
	}
	fprintf(out,"</map>");
	footer();
}


void make_dot_png(Database& db,Query& q,char *sql,db::Dbs *sdb)
{
	FILE *fil;
	struct stat fs;
	char filename[200];

	sprintf(filename,"/tmp/image%ld.png",sdb -> num);
	stat(filename, &fs);

	fprintf(out,"Content-type: image/png\n");
	fprintf(out,"Content-length: %d\n",fs.st_size);
	fprintf(out,"\n");

	if ((fil = fopen(filename,"rb")) != NULL)
	{
		char buf[1000];
		size_t n = fread(buf,1,1000,fil);
		while (n > 0)
		{
			fwrite(buf,1,n,out);
			//
			n = fread(buf,1,1000,fil);
		}
		fclose(fil);
	}
/*
	sprintf(filename,"/tmp/%ld.dot",sdb -> num);
	unlink( filename );
	sprintf(filename,"/tmp/image%ld.png",sdb -> num);
	unlink( filename );
*/
}


